"""Parse My Clippings highlights from kindle."""
import re
from typing import Any
from typing import List

from textwrap3 import wrap

# import dateparser

regexp_author = re.compile(r"\((.*?)\)", re.IGNORECASE)
regexp_map = {
    "es": {
        "regexp_page": re.compile(r"página ([0-9]+)", re.IGNORECASE),
        "regexp_position_high": re.compile(
            r"posición ([0-9]+)-([0-9]+)", re.IGNORECASE
        ),
        "regexp_position_note": re.compile(r"posición ([0-9]+)", re.IGNORECASE),
        "regexp_date": re.compile(r"añadido el\s+(.+)$", re.IGNORECASE),
        "regexp_note": re.compile(r"nota", re.IGNORECASE),
        "regexp_highlight": re.compile(r"subrayado", re.IGNORECASE),
        "regexp_mark": re.compile(r"marcador", re.IGNORECASE),
    },
    "en": {
        "regexp_page": re.compile(r"page ([0-9]+)", re.IGNORECASE),
        "regexp_position_high": re.compile(
            r"location ([0-9]+)-([0-9]+)", re.IGNORECASE
        ),
        "regexp_position_note": re.compile(r"location ([0-9]+)", re.IGNORECASE),
        "regexp_date": re.compile(r"added on\s+(.+)$", re.IGNORECASE),
        "regexp_note": re.compile(r"note", re.IGNORECASE),
        "regexp_highlight": re.compile(r"highlight", re.IGNORECASE),
        "regexp_mark": re.compile(r"bookmark", re.IGNORECASE),
    },
}


class Book:
    """Book information with a list of clippings."""

    def __init__(self, title: str, author: str) -> None:
        """Basic initialization of a Book."""
        self.author = author
        self.title = title
        self.clippings: List[Any] = []

    def add_clipping(self, clipping: Any) -> None:
        """Add a clipping to this list."""
        if clipping:
            self.clippings.append(clipping)

    def get_highlights(self) -> List[Any]:
        """Get all hightlights of the book."""
        return [m for m in self.clippings if m.type == "HIGHLIGHT"]

    def get_marks(self) -> List[Any]:
        """Get all marks of the book."""
        return [m for m in self.clippings if m.type == "MARK"]

    def get_notes(self) -> List[Any]:
        """Get all notes of the book."""
        return [m for m in self.clippings if m.type == "NOTE"]

    def __str__(self) -> str:
        """Generate a string representation."""
        return (
            f"Title:{self.title}\tAuthor:{self.author}\tClippings:{len(self.clippings)}"
        )


class Clipping:
    """Information about a single clipping."""

    def __init__(self, title: str, author: str, metadata: str, content: str) -> None:
        """Basic initialization for a clipping."""
        self.title = title
        self.author = author
        self.metadata = metadata
        self.content = content
        self.page: int = -1
        self.position_end: int = -1
        self.position_start: int = -1
        self.date: Any = None
        self.type: str = "NONE"

    def get_header(self) -> str:
        """Get a header for the clipping."""
        return self.metadata.replace("- ", "** ")

    def get_wrap_content(self) -> str:
        """Get a word wrap text from content."""
        clean_text = self.content.replace("\n", " ")
        return "\n".join(wrap(clean_text, 80))

    def parse_metadata(self, lang: str) -> None:
        """Extract information from metadata line."""
        if regexp_map[lang]["regexp_highlight"].search(self.metadata):
            self.type = "HIGHLIGHT"
            match_page = regexp_map[lang]["regexp_page"].search(self.metadata)
            if match_page:
                self.page = int(match_page.group(1))
            match_position = regexp_map[lang]["regexp_position_high"].search(
                self.metadata
            )
            if match_position:
                self.position_start = int(match_position.group(1))
                self.position_end = int(match_position.group(2))
        if regexp_map[lang]["regexp_note"].search(self.metadata):
            self.type = "NOTE"
            match_position = regexp_map[lang]["regexp_position_note"].search(
                self.metadata
            )
            if match_position:
                self.position_start = int(match_position.group(1))
        if regexp_map[lang]["regexp_mark"].search(self.metadata):
            self.type = "MARK"
            match_position = regexp_map[lang]["regexp_position_note"].search(
                self.metadata
            )
            if match_position:
                self.position_start = int(match_position.group(1))
        # match_date = regexp_date.search(metadata)
        # if match_date:
        #     h.date = dateparser.parse(match_date.group(1))

    @classmethod
    def parse_single_highlight(cls, highlight_string: str) -> Any:
        """Parse a single clipping."""
        splitted_string = highlight_string.split("\n")
        if len(splitted_string) < 4:
            return None
        author_line = splitted_string[0]
        metadata = splitted_string[1]
        content = splitted_string[3]
        # regex = r"\((.*?)\)"
        # Solving problems with titles with parenthesis
        match_author = regexp_author.findall(author_line)
        if len(match_author) == 0:
            return None
        for a in match_author:
            author = a
            title = author_line.replace(f"({a})", "").strip()
        if not title:
            return None
        return Clipping(title, author, metadata, content)


def parser_my_clippings_data(my_clipping_data: str, language: str) -> List[Any]:
    """Parser 'My Clippings.txt' data."""
    clipping_separator = "\n==========\n"

    clippings = my_clipping_data.split(clipping_separator)

    book_title_list = []
    library = []
    for raw_string in clippings:
        clipping = Clipping.parse_single_highlight(raw_string)

        if not clipping:
            continue

        clipping.parse_metadata(language)
        if clipping.title not in book_title_list:
            book = Book(clipping.title, clipping.author)
            book.add_clipping(clipping)
            book_title_list.append(clipping.title)
            library.append(book)
        else:
            for book in library:
                if book.title == clipping.title:
                    book.add_clipping(clipping)

    return library



"""Exporter module for books and clippings."""
from typing import Any

from slugify import slugify


def write_book(book: Any, directory: Any) -> None:
    """Write all clippings in a file within the output folder."""
    if not book.title or len(book.clippings) == 0:
        print(f"Not writting {book.title} because name is {len(book.clippings)}")
        return None
    clean_title = "".join(
        [c for c in book.title if c.isalpha() or c.isdigit() or c == " "]
    ).rstrip()

    book.clippings.sort(
        key=lambda h: h.position_start if h.position_start else h.page if h.page else 0
    )

    if not directory.is_dir():
        return None

    path_file = directory / f"{slugify(clean_title)}.org"
    with path_file.open(mode="w") as file:
        file.write(f"#+title: {clean_title}\n")
        file.write(f"#+author: {book.author}\n")
        file.write("\n")

        notes = book.get_notes()
        if len(notes) > 0:
            file.write("* Notes")
            file.write("\n")
            for h in notes:
                file.write(f"\n{h.get_header()}\n")
                file.write(f"\n{h.get_wrap_content()}\n")

        marks = book.get_marks()
        if len(marks) > 0:
            file.write("* Bookmarks")
            file.write("\n")
            for h in marks:
                file.write(f"\n{h.get_header()}\n")

        highlights = book.get_highlights()
        if len(highlights) > 0:
            file.write("* Highlights")
            file.write("\n")
            for h in highlights:
                file.write(f"\n{h.get_header()}\n")
                file.write(
                    f"\n#+caption: {clean_title} \
                    ({book.author}) in position {h.position_start}\n"
                )
                file.write("#+begin_quote\n")
                file.write(h.get_wrap_content())
                file.write("\n#+end_quote\n")

        file.close()



"""Command-line interface."""
from pathlib import Path

import click

from pyclip2org import exporter
from pyclip2org import parser


@click.command()
@click.version_option()
@click.option(
    "-o",
    "--output-dir",
    default="/tmp/pyclip2org",
    help="Output directory, '/tmp/pyclip2org' by default",
)
@click.option(
    "-c", "--my-clippings", required=True, help="'My Clippings.txt' file path"
)
@click.option(
    "-l",
    "--language",
    default="en",
    help="'My Clippings.txt' metadata language, (es|en)",
)
def main(my_clippings: str, output_dir: str, language: str) -> None:
    """My Clipping to org-mode notes."""
    directory_path = Path(output_dir)

    if language != "en" and language != "es":
        raise click.UsageError(f"Incorrect language {language}, must be 'en' or 'es'")

    if not directory_path.exists():
        directory_path.mkdir(parents=True)

    if not directory_path.is_dir():
        raise click.UsageError("Output directory is not a folder")

    with open(my_clippings, "r") as file:
        data = file.read()

    library = parser.parser_my_clippings_data(data, language)
    for book in library:
        click.echo(f"Writing {book.title} with {len(book.clippings)} highlights")
        exporter.write_book(book, directory_path)


if __name__ == "__main__":
    main(prog_name="pyclip2org")  # pragma: no cover



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['pyclip2org']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.0,<8.0', 'python-slugify>=4.0.1,<5.0.0', 'textwrap3>=0.9.2,<0.10.0']

entry_points = \
{'console_scripts': ['pyclip2org = pyclip2org.__main__:main']}

setup_kwargs = {
    'name': 'pyclip2org',
    'version': '0.1.1',
    'description': 'My Clipping to org-mode notes',
    'long_description': "My Clipping to org-mode notes\n=============================\n\n|PyPI| |Python Version| |License|\n\n|Read the Docs| |Tests| |Codecov|\n\n|pre-commit| |Black|\n\n.. |PyPI| image:: https://img.shields.io/pypi/v/pyclip2org.svg\n   :target: https://pypi.org/project/pyclip2org/\n   :alt: PyPI\n.. |Python Version| image:: https://img.shields.io/pypi/pyversions/pyclip2org\n   :target: https://pypi.org/project/pyclip2org\n   :alt: Python Version\n.. |License| image:: https://img.shields.io/pypi/l/pyclip2org\n   :target: https://opensource.org/licenses/MIT\n   :alt: License\n.. |Read the Docs| image:: https://img.shields.io/readthedocs/pyclip2org/latest.svg?label=Read%20the%20Docs\n   :target: https://pyclip2org.readthedocs.io/\n   :alt: Read the documentation at https://pyclip2org.readthedocs.io/\n.. |Tests| image:: https://github.com/ppalazon/pyclip2org/workflows/Tests/badge.svg\n   :target: https://github.com/ppalazon/pyclip2org/actions?workflow=Tests\n   :alt: Tests\n.. |Codecov| image:: https://codecov.io/gh/ppalazon/pyclip2org/branch/master/graph/badge.svg\n   :target: https://codecov.io/gh/ppalazon/pyclip2org\n   :alt: Codecov\n.. |pre-commit| image:: https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white\n   :target: https://github.com/pre-commit/pre-commit\n   :alt: pre-commit\n.. |Black| image:: https://img.shields.io/badge/code%20style-black-000000.svg\n   :target: https://github.com/psf/black\n   :alt: Black\n\n\nFeatures\n--------\n\n* Parse 'My Clippings.txt' generated file\n* Extract highlights, marks and notes\n* Export to org-mode files separated by books\n\nRequirements\n------------\n\n* You will need a 'My Clippings.txt' file in English or Spanish\n\n\nInstallation\n------------\n\nYou can install *My Clipping to org-mode notes* via pip_ from PyPI_:\n\n.. code:: console\n\n   $ pip install pyclip2org\n\n\nUsage\n-----\n\nOnce you've installed, you can execute\n\n.. code:: console\n\n   $ pyclip2org -l en -o ~/org/kindle -c /media/Kindle/documents/My\\ Clippings.txt\n\nPlease see the `Command-line Reference <Usage_>`_ for details.\n\n\nContributing\n------------\n\nContributions are very welcome.\nTo learn more, see the `Contributor Guide`_.\n\n\nLicense\n-------\n\nDistributed under the terms of the MIT_ license,\n*My Clipping to org-mode notes* is free and open source software.\n\n\nIssues\n------\n\nIf you encounter any problems,\nplease `file an issue`_ along with a detailed description.\n\n\nCredits\n-------\n\nThis project was generated from `@cjolowicz`_'s `Hypermodern Python Cookiecutter`_ template.\nThis project is based on\n`Managing kindle highlights with Python and GitHub <https://duarteocarmo.com/blog/managing-kindle-highlights-with-python-and-github.html>`_\n\n.. _@cjolowicz: https://github.com/cjolowicz\n.. _Cookiecutter: https://github.com/audreyr/cookiecutter\n.. _MIT: http://opensource.org/licenses/MIT\n.. _PyPI: https://pypi.org/\n.. _Hypermodern Python Cookiecutter: https://github.com/cjolowicz/cookiecutter-hypermodern-python\n.. _file an issue: https://github.com/ppalazon/pyclip2org/issues\n.. _pip: https://pip.pypa.io/\n.. github-only\n.. _Contributor Guide: CONTRIBUTING.rst\n.. _Usage: https://pyclip2org.readthedocs.io/en/latest/usage.html\n",
    'author': 'Pablo Palazon',
    'author_email': 'pablo.palazon@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/ppalazon/pyclip2org',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
